@@ -19,6 +19,7 @@ import hashlib
 import logging
 import math
 
+from keystoneclient import exceptions as keystone_exc
 from keystoneclient import service_catalog as keystone_sc
 from oslo_config import cfg
 from oslo_utils import encodeutils
@@ -32,7 +33,12 @@ try:
 except ImportError:
     swiftclient = None
 
+from keystoneclient.auth.identity import v3 as ks_v3
+from keystoneclient import session as ks_session
+from keystoneclient.v3 import client as ks_client
+
 import glance_store
+from glance_store._drivers.swift import connection_manager
 from glance_store._drivers.swift import utils as sutils
 from glance_store import capabilities
 from glance_store import driver
@@ -76,38 +82,170 @@ _SWIFT_OPTS = [
                       'swift_store_auth_version is 2.')),
     cfg.StrOpt('swift_store_container',
                default=DEFAULT_CONTAINER,
-               help=_('Container within the account that the account should '
-                      'use for storing images in Swift when using single '
-                      'container mode. In multiple container mode, this will '
-                      'be the prefix for all containers.')),
+               help=_("""
+Name of single container to store images/name prefix for multiple containers
+
+When a single container is being used to store images, this configuration
+option indicates the container within the Glance account to be used for
+storing all images. When multiple containers are used to store images, this
+will be the name prefix for all containers. Usage of single/multiple
+containers can be controlled using the configuration option
+``swift_store_multiple_containers_seed``.
+
+When using multiple containers, the containers will be named after the value
+set for this configuration option with the first N chars of the image UUID
+as the suffix delimited by an underscore (where N is specified by
+``swift_store_multiple_containers_seed``).
+
+Example: if the seed is set to 3 and swift_store_container = ``glance``, then
+an image with UUID ``fdae39a1-bac5-4238-aba4-69bcc726e848`` would be placed in
+the container ``glance_fda``. All dashes in the UUID are included when
+creating the container name but do not count toward the character limit, so
+when N=10 the container name would be ``glance_fdae39a1-ba.``
+
+Services which consume this:
+    * None (directly consumed by the Swift driver)
+
+Possible values:
+    * If using single container, this configuration option can be any string
+      that is a valid swift container name in Glance's Swift account
+    * If using multiple containers, this configuration option can be any
+      string as long as it satisfies the container naming rules enforced by
+      Swift. The value of ``swift_store_multiple_containers_seed`` should be
+      taken into account as well.
+
+Related options:
+    * ``swift_store_multiple_containers_seed``
+    * ``swift_store_multi_tenant``
+    * ``swift_store_create_container_on_put``
+
+""")),
     cfg.IntOpt('swift_store_large_object_size',
-               default=DEFAULT_LARGE_OBJECT_SIZE,
-               help=_('The size, in MB, that Glance will start chunking image '
-                      'files and do a large object manifest in Swift.')),
+               default=DEFAULT_LARGE_OBJECT_SIZE, min=1,
+               help=_("""
+The size threshold, in MB, after which Glance will start segmenting image data.
+
+Swift has an upper limit on the size of a single uploaded object. By default,
+this is 5GB. To upload objects bigger than this limit, objects are segmented
+into multiple smaller objects that are tied together with a manifest file.
+For more detail, refer to
+http://docs.openstack.org/developer/swift/overview_large_objects.html
+
+This configuration option specifies the size threshold over which the Swift
+driver will start segmenting image data into multiple smaller files.
+Currently, the Swift driver only supports creating Dynamic Large Objects.
+
+NOTE: This should be set by taking into account the large object limit
+enforced by the Swift cluster in consideration.
+
+Services which consume this:
+    * None (directly consumed by the Swift driver)
+
+Possible values:
+    * A positive integer that is less than or equal to the large object limit
+      enforced by the Swift cluster in consideration.
+
+Related options:
+    * ``swift_store_large_object_chunk_size``
+
+""")),
     cfg.IntOpt('swift_store_large_object_chunk_size',
-               default=DEFAULT_LARGE_OBJECT_CHUNK_SIZE,
-               help=_('The amount of data written to a temporary '
-                      'disk buffer during the process of chunking '
-                      'the image file.')),
+               default=DEFAULT_LARGE_OBJECT_CHUNK_SIZE, min=1,
+               help=_("""
+The maximum size, in MB, of the segments when image data is segmented.
+
+When image data is segmented to upload images that are larger than the limit
+enforced by the Swift cluster, image data is broken into segments that are no
+bigger than the size specified by this configuration option.
+Refer to ``swift_store_large_object_size`` for more detail.
+
+For example: if ``swift_store_large_object_size`` is 5GB and
+``swift_store_large_object_chunk_size`` is 1GB, an image of size 6.2GB will be
+segmented into 7 segments where the first six segments will be 1GB in size and
+the seventh segment will be 0.2GB.
+
+Services which consume this:
+    * None (directly consumed by the Swift driver)
+
+Possible values:
+    * A positive integer that is less than or equal to the large object limit
+      enforced by Swift cluster in consideration.
+
+Related options:
+    * ``swift_store_large_object_size``
+
+""")),
     cfg.BoolOpt('swift_store_create_container_on_put', default=False,
-                help=_('A boolean value that determines if we create the '
-                       'container if it does not exist.')),
+                help=_("""
+Create container, if it doesn't already exist, when uploading image.
+
+At the time of uploading an image, if the corresponding container doesn't
+exist, it will be created provided this configuration option is set to True.
+By default, it won't be created. This behavior is applicable for both single
+and multiple containers mode.
+
+Services which consume this:
+    * None (directly consumed by the Swift driver)
+
+Possible values:
+    * True
+    * False
+
+Related options:
+    * None
+
+""")),
     cfg.BoolOpt('swift_store_multi_tenant', default=False,
-                help=_('If set to True, enables multi-tenant storage '
-                       'mode which causes Glance images to be stored in '
-                       'tenant specific Swift accounts.')),
+                help=_("""
+Store images in tenant's Swift account.
+
+This enables multi-tenant storage mode which causes Glance images to be stored
+in tenant specific Swift accounts. If this is disabled, Glance stores all
+images in its own account. More details multi-tenant store can be found at
+https://wiki.openstack.org/wiki/GlanceSwiftTenantSpecificStorage
+
+Services which consume this:
+    * None (directly consumed by the Swift driver)
+
+Possible values:
+    * True
+    * False
+
+Related options:
+    * None
+
+""")),
     cfg.IntOpt('swift_store_multiple_containers_seed',
-               default=0,
-               help=_('When set to 0, a single-tenant store will only use one '
-                      'container to store all images. When set to an integer '
-                      'value between 1 and 32, a single-tenant store will use '
-                      'multiple containers to store images, and this value '
-                      'will determine how many containers are created.'
-                      'Used only when swift_store_multi_tenant is disabled. '
-                      'The total number of containers that will be used is '
-                      'equal to 16^N, so if this config option is set to 2, '
-                      'then 16^2=256 containers will be used to store images.'
-                      )),
+               default=0, min=0, max=32,
+               help=_("""
+Seed indicating the number of containers to use for storing images.
+
+When using a single-tenant store, images can be stored in one or more than one
+containers. When set to 0, all images will be stored in one single container.
+When set to an integer value between 1 and 32, multiple containers will be
+used to store images. This configuration option will determine how many
+containers are created. The total number of containers that will be used is
+equal to 16^N, so if this config option is set to 2, then 16^2=256 containers
+will be used to store images.
+
+Please refer to ``swift_store_container`` for more detail on the naming
+convention. More detail about using multiple containers can be found at
+https://specs.openstack.org/openstack/glance-specs/specs/kilo/swift-store-multiple-containers.html
+
+NOTE: This is used only when swift_store_multi_tenant is disabled.
+
+Services which consume this:
+    * None (directly consumed only by the Swift driver)
+
+Possible values:
+    * A non-negative integer less than or equal to 32
+
+Related options:
+    * ``swift_store_container``
+    * ``swift_store_multi_tenant``
+    * ``swift_store_create_container_on_put``
+
+""")),
     cfg.ListOpt('swift_store_admin_tenants', default=[],
                 help=_('A list of tenants that will be granted read/write '
                        'access on all Swift containers created by Glance in '
@@ -119,11 +257,26 @@ _SWIFT_OPTS = [
                        'compressed format, eg qcow2.')),
     cfg.IntOpt('swift_store_retry_get_count', default=0,
                help=_('The number of times a Swift download will be retried '
-                      'before the request fails.'))
+                      'before the request fails.')),
+    cfg.IntOpt('swift_store_expire_soon_interval', default=60,
+               help=_('The period of time (in seconds) before token '
+                      'expiration when glance_store will try to request new '
+                      'user token. Default value 60 sec means that if token '
+                      'is going to expire in 1 min then glance_store requests '
+                      'new user token.')),
+    cfg.BoolOpt('swift_store_use_trusts', default=True,
+                help=_('If set to True create a trust for each add/get '
+                       'request to Multi-tenant store in order to prevent '
+                       'authentication token to be expired during '
+                       'uploading/downloading data. If set to False then user '
+                       'token is used for Swift connection (so no overhead on '
+                       'trust creation). Please note that this '
+                       'option is considered only and only if '
+                       'swift_store_multi_tenant=True'))
 ]
 
 
-def swift_retry_iter(resp_iter, length, store, location, context):
+def swift_retry_iter(resp_iter, length, store, location, manager):
     if not length and isinstance(resp_iter, six.BytesIO):
         if six.PY3:
             # On Python 3, io.BytesIO does not have a len attribute, instead
@@ -168,9 +321,9 @@ def swift_retry_iter(resp_iter, length, store, location, context):
                           'max_retries': retry_count,
                           'start': bytes_read,
                           'end': length})
-                (_resp_headers, resp_iter) = store._get_object(location, None,
-                                                               bytes_read,
-                                                               context=context)
+                (_resp_headers, resp_iter) = store._get_object(location,
+                                                               manager,
+                                                               bytes_read)
         else:
             break
 
@@ -423,16 +576,14 @@ class BaseStore(driver.Store):
                                                    reason=msg)
         super(BaseStore, self).configure(re_raise_bsc=re_raise_bsc)
 
-    def _get_object(self, location, connection=None, start=None, context=None):
-        if not connection:
-            connection = self.get_connection(location, context=context)
+    def _get_object(self, location, manager, start=None):
         headers = {}
         if start is not None:
             bytes_range = 'bytes=%d-' % start
             headers = {'Range': bytes_range}
 
         try:
-            resp_headers, resp_body = connection.get_object(
+            resp_headers, resp_body = manager.get_connection().get_object(
                 location.container, location.obj,
                 resp_chunk_size=self.CHUNKSIZE, headers=headers)
         except swiftclient.ClientException as e:
@@ -449,21 +600,26 @@ class BaseStore(driver.Store):
     def get(self, location, connection=None,
             offset=0, chunk_size=None, context=None):
         location = location.store_location
-        (resp_headers, resp_body) = self._get_object(location, connection,
-                                                     context=context)
+        # initialize manager to receive valid connections
+        allow_retry = \
+            self.conf.glance_store.swift_store_retry_get_count > 0
+        with get_manager_for_store(self, location, context,
+                                   allow_reauth=allow_retry) as manager:
+            (resp_headers, resp_body) = self._get_object(location,
+                                                         manager=manager)
 
-        class ResponseIndexable(glance_store.Indexable):
-            def another(self):
-                try:
-                    return next(self.wrapped)
-                except StopIteration:
-                    return ''
+            class ResponseIndexable(glance_store.Indexable):
+                def another(self):
+                    try:
+                        return next(self.wrapped)
+                    except StopIteration:
+                        return ''
 
-        length = int(resp_headers.get('content-length', 0))
-        if self.conf.glance_store.swift_store_retry_get_count > 0:
-            resp_body = swift_retry_iter(resp_body, length,
-                                         self, location, context)
-        return (ResponseIndexable(resp_body, length), length)
+            length = int(resp_headers.get('content-length', 0))
+            if allow_retry:
+                resp_body = swift_retry_iter(resp_body, length,
+                                             self, location, manager=manager)
+            return ResponseIndexable(resp_body, length), length
 
     def get_size(self, location, connection=None, context=None):
         location = location.store_location
@@ -499,129 +655,143 @@ class BaseStore(driver.Store):
 
     @capabilities.check
     def add(self, image_id, image_file, image_size,
-            connection=None, context=None, verifier=None):
+            context=None, verifier=None):
         location = self.create_location(image_id, context=context)
-        if not connection:
-            connection = self.get_connection(location, context=context)
+        # initialize a manager with re-auth if image need to be splitted
+        need_chunks = (image_size == 0) or (
+            image_size >= self.large_object_size)
+        with get_manager_for_store(self, location, context,
+                                   allow_reauth=need_chunks) as manager:
 
-        self._create_container_if_missing(location.container, connection)
+            self._create_container_if_missing(location.container,
+                                              manager.get_connection())
 
-        LOG.debug("Adding image object '%(obj_name)s' "
-                  "to Swift" % dict(obj_name=location.obj))
-        try:
-            if image_size > 0 and image_size < self.large_object_size:
-                # Image size is known, and is less than large_object_size.
-                # Send to Swift with regular PUT.
-                obj_etag = connection.put_object(location.container,
-                                                 location.obj, image_file,
-                                                 content_length=image_size)
-            else:
-                # Write the image into Swift in chunks.
-                chunk_id = 1
-                if image_size > 0:
-                    total_chunks = str(int(
-                        math.ceil(float(image_size) /
-                                  float(self.large_object_chunk_size))))
-                else:
-                    # image_size == 0 is when we don't know the size
-                    # of the image. This can occur with older clients
-                    # that don't inspect the payload size.
-                    LOG.debug("Cannot determine image size. Adding as a "
-                              "segmented object to Swift.")
-                    total_chunks = '?'
-
-                checksum = hashlib.md5()
-                written_chunks = []
-                combined_chunks_size = 0
-                while True:
-                    chunk_size = self.large_object_chunk_size
-                    if image_size == 0:
-                        content_length = None
+            LOG.debug("Adding image object '%(obj_name)s' "
+                      "to Swift" % dict(obj_name=location.obj))
+            try:
+                if not need_chunks:
+                    # Image size is known, and is less than large_object_size.
+                    # Send to Swift with regular PUT.
+                    if verifier:
+                        checksum = hashlib.md5()
+                        reader = ChunkReader(image_file, checksum,
+                                             image_size, verifier)
+                        obj_etag = manager.get_connection().put_object(
+                            location.container, location.obj,
+                            reader, content_length=image_size)
                     else:
-                        left = image_size - combined_chunks_size
-                        if left == 0:
+                        obj_etag = manager.get_connection().put_object(
+                            location.container, location.obj,
+                            image_file, content_length=image_size)
+                else:
+                    # Write the image into Swift in chunks.
+                    chunk_id = 1
+                    if image_size > 0:
+                        total_chunks = str(int(
+                            math.ceil(float(image_size) /
+                                      float(self.large_object_chunk_size))))
+                    else:
+                        # image_size == 0 is when we don't know the size
+                        # of the image. This can occur with older clients
+                        # that don't inspect the payload size.
+                        LOG.debug("Cannot determine image size. Adding as a "
+                                  "segmented object to Swift.")
+                        total_chunks = '?'
+
+                    checksum = hashlib.md5()
+                    written_chunks = []
+                    combined_chunks_size = 0
+                    while True:
+                        chunk_size = self.large_object_chunk_size
+                        if image_size == 0:
+                            content_length = None
+                        else:
+                            left = image_size - combined_chunks_size
+                            if left == 0:
+                                break
+                            if chunk_size > left:
+                                chunk_size = left
+                            content_length = chunk_size
+
+                        chunk_name = "%s-%05d" % (location.obj, chunk_id)
+                        reader = ChunkReader(image_file, checksum, chunk_size,
+                                             verifier)
+                        if reader.is_zero_size is True:
+                            LOG.debug('Not writing zero-length chunk.')
                             break
-                        if chunk_size > left:
-                            chunk_size = left
-                        content_length = chunk_size
+                        try:
+                            chunk_etag = manager.get_connection().put_object(
+                                location.container, chunk_name, reader,
+                                content_length=content_length)
+                            written_chunks.append(chunk_name)
+                        except Exception:
+                            # Delete orphaned segments from swift backend
+                            with excutils.save_and_reraise_exception():
+                                LOG.exception(_("Error during chunked upload "
+                                                "to backend, deleting stale "
+                                                "chunks"))
+                                self._delete_stale_chunks(
+                                    manager.get_connection(),
+                                    location.container,
+                                    written_chunks)
 
-                    chunk_name = "%s-%05d" % (location.obj, chunk_id)
-                    reader = ChunkReader(image_file, checksum, chunk_size,
-                                         verifier)
-                    if reader.is_zero_size is True:
-                        LOG.debug('Not writing zero-length chunk.')
-                        break
-                    try:
-                        chunk_etag = connection.put_object(
-                            location.container, chunk_name, reader,
-                            content_length=content_length)
-                        written_chunks.append(chunk_name)
-                    except Exception:
-                        # Delete orphaned segments from swift backend
-                        with excutils.save_and_reraise_exception():
-                            LOG.exception(_("Error during chunked upload to "
-                                            "backend, deleting stale chunks"))
-                            self._delete_stale_chunks(connection,
-                                                      location.container,
-                                                      written_chunks)
+                        bytes_read = reader.bytes_read
+                        msg = ("Wrote chunk %(chunk_name)s (%(chunk_id)d/"
+                               "%(total_chunks)s) of length %(bytes_read)d "
+                               "to Swift returning MD5 of content: "
+                               "%(chunk_etag)s" %
+                               {'chunk_name': chunk_name,
+                                'chunk_id': chunk_id,
+                                'total_chunks': total_chunks,
+                                'bytes_read': bytes_read,
+                                'chunk_etag': chunk_etag})
+                        LOG.debug(msg)
 
-                    bytes_read = reader.bytes_read
-                    msg = ("Wrote chunk %(chunk_name)s (%(chunk_id)d/"
-                           "%(total_chunks)s) of length %(bytes_read)d "
-                           "to Swift returning MD5 of content: "
-                           "%(chunk_etag)s" %
-                           {'chunk_name': chunk_name,
-                            'chunk_id': chunk_id,
-                            'total_chunks': total_chunks,
-                            'bytes_read': bytes_read,
-                            'chunk_etag': chunk_etag})
-                    LOG.debug(msg)
+                        chunk_id += 1
+                        combined_chunks_size += bytes_read
 
-                    chunk_id += 1
-                    combined_chunks_size += bytes_read
+                    # In the case we have been given an unknown image size,
+                    # set the size to the total size of the combined chunks.
+                    if image_size == 0:
+                        image_size = combined_chunks_size
 
-                # In the case we have been given an unknown image size,
-                # set the size to the total size of the combined chunks.
-                if image_size == 0:
-                    image_size = combined_chunks_size
+                    # Now we write the object manifest and return the
+                    # manifest's etag...
+                    manifest = "%s/%s-" % (location.container, location.obj)
+                    headers = {'ETag': hashlib.md5(b"").hexdigest(),
+                               'X-Object-Manifest': manifest}
 
-                # Now we write the object manifest and return the
-                # manifest's etag...
-                manifest = "%s/%s-" % (location.container, location.obj)
-                headers = {'ETag': hashlib.md5(b"").hexdigest(),
-                           'X-Object-Manifest': manifest}
+                    # The ETag returned for the manifest is actually the
+                    # MD5 hash of the concatenated checksums of the strings
+                    # of each chunk...so we ignore this result in favour of
+                    # the MD5 of the entire image file contents, so that
+                    # users can verify the image file contents accordingly
+                    manager.get_connection().put_object(location.container,
+                                                        location.obj,
+                                                        None, headers=headers)
+                    obj_etag = checksum.hexdigest()
 
-                # The ETag returned for the manifest is actually the
-                # MD5 hash of the concatenated checksums of the strings
-                # of each chunk...so we ignore this result in favour of
-                # the MD5 of the entire image file contents, so that
-                # users can verify the image file contents accordingly
-                connection.put_object(location.container, location.obj,
-                                      None, headers=headers)
-                obj_etag = checksum.hexdigest()
+                # NOTE: We return the user and key here! Have to because
+                # location is used by the API server to return the actual
+                # image data. We *really* should consider NOT returning
+                # the location attribute from GET /images/<ID> and
+                # GET /images/details
+                if sutils.is_multiple_swift_store_accounts_enabled(self.conf):
+                    include_creds = False
+                else:
+                    include_creds = True
+                return (location.get_uri(credentials_included=include_creds),
+                        image_size, obj_etag, {})
+            except swiftclient.ClientException as e:
+                if e.http_status == http_client.CONFLICT:
+                    msg = _("Swift already has an image at this location")
+                    raise exceptions.Duplicate(message=msg)
 
-            # NOTE: We return the user and key here! Have to because
-            # location is used by the API server to return the actual
-            # image data. We *really* should consider NOT returning
-            # the location attribute from GET /images/<ID> and
-            # GET /images/details
-            if sutils.is_multiple_swift_store_accounts_enabled(self.conf):
-                include_creds = False
-            else:
-                include_creds = True
-
-            return (location.get_uri(credentials_included=include_creds),
-                    image_size, obj_etag, {})
-        except swiftclient.ClientException as e:
-            if e.http_status == http_client.CONFLICT:
-                msg = _("Swift already has an image at this location")
-                raise exceptions.Duplicate(message=msg)
-
-            msg = (_(u"Failed to add object to Swift.\n"
-                     "Got error from Swift: %s.")
-                   % encodeutils.exception_to_unicode(e))
-            LOG.error(msg)
-            raise glance_store.BackendException(msg)
+                msg = (_(u"Failed to add object to Swift.\n"
+                         "Got error from Swift: %s.")
+                       % encodeutils.exception_to_unicode(e))
+                LOG.error(msg)
+                raise glance_store.BackendException(msg)
 
     @capabilities.check
     def delete(self, location, connection=None, context=None):
@@ -719,6 +889,35 @@ class BaseStore(driver.Store):
     def create_location(self, image_id, context=None):
         raise NotImplementedError()
 
+    def init_client(self, location, context=None):
+        """Initialize and return client to authorize against keystone
+
+        The method invariant is the following: it always returns Keystone
+        client that can be used to receive fresh token in any time. Otherwise
+        it raises appropriate exception.
+        :param location: swift location data
+        :param context: user context (it is not required if user grants are
+        specified for single tenant store)
+        :return correctly initialized keystone client
+        """
+        raise NotImplementedError()
+
+    def get_store_connection(self, auth_token, storage_url):
+        """Get initialized swift connection
+
+        :param auth_token: auth token
+        :param storage_url: swift storage url
+        :return: swiftclient connection that allows to request container and
+        others
+        """
+        # initialize a connection
+        return swiftclient.Connection(
+            preauthurl=storage_url,
+            preauthtoken=auth_token,
+            insecure=self.insecure,
+            ssl_compression=self.ssl_compression,
+            cacert=self.cacert)
+
 
 class SingleTenantStore(BaseStore):
     EXAMPLE_URL = "swift://<USER>:<KEY>@<AUTH_ADDRESS>/<CONTAINER>/<FILE>"
@@ -856,6 +1055,39 @@ class SingleTenantStore(BaseStore):
             auth_version=self.auth_version, os_options=os_options,
             ssl_compression=self.ssl_compression, cacert=self.cacert)
 
+    def init_client(self, location, context=None):
+        """Initialize keystone client with swift service user credentials"""
+        # prepare swift admin credentials
+        if not location.user:
+            reason = _("Location is missing user:password information.")
+            LOG.info(reason)
+            raise exceptions.BadStoreUri(message=reason)
+
+        auth_url = location.swift_url
+        if not auth_url.endswith('/'):
+            auth_url += '/'
+
+        try:
+            tenant_name, user = location.user.split(':')
+        except ValueError:
+            reason = (_("Badly formed tenant:user '%(user)s' in "
+                        "Swift URI") % {'user': location.user})
+            LOG.info(reason)
+            raise exceptions.BadStoreUri(message=reason)
+
+        # initialize a keystone plugin for swift admin with creds
+        password = ks_v3.Password(auth_url=auth_url,
+                                  username=user,
+                                  password=location.key,
+                                  project_name=tenant_name,
+                                  user_domain_id=self.user_domain_id,
+                                  user_domain_name=self.user_domain_name,
+                                  project_domain_id=self.project_domain_id,
+                                  project_domain_name=self.project_domain_name)
+        sess = ks_session.Session(auth=password)
+
+        return ks_client.Client(session=sess)
+
 
 class MultiTenantStore(BaseStore):
     EXAMPLE_URL = "swift://<SWIFT_URL>/<CONTAINER>/<FILE>"
@@ -937,13 +1169,87 @@ class MultiTenantStore(BaseStore):
         return StoreLocation(specs, self.conf)
 
     def get_connection(self, location, context=None):
+        try:
+            storage_url = self._get_endpoint(context)
+        except (exceptions.BadStoreConfiguration,
+                keystone_exc.EndpointNotFound) as e:
+            LOG.debug("Cannot obtain swift endpoint url from Service Catalog: "
+                      "%s. Use url stored in database.", e)
+            storage_url = location.swift_url
         return swiftclient.Connection(
-            preauthurl=location.swift_url,
+            preauthurl=storage_url,
             preauthtoken=context.auth_token,
             insecure=self.insecure,
             ssl_compression=self.ssl_compression,
             cacert=self.cacert)
 
+    def init_client(self, location, context=None):
+        # read client parameters from config files
+        ref_params = sutils.SwiftParams(self.conf).params
+        default_ref = self.conf.glance_store.default_swift_reference
+        default_swift_reference = ref_params.get(default_ref)
+        if not default_swift_reference:
+            reason = _("default_swift_reference %s is required.") % default_ref
+            LOG.error(reason)
+            raise exceptions.BadStoreConfiguration(message=reason)
+
+        auth_address = default_swift_reference.get('auth_address')
+        user = default_swift_reference.get('user')
+        key = default_swift_reference.get('key')
+        user_domain_id = default_swift_reference.get('user_domain_id')
+        user_domain_name = default_swift_reference.get('user_domain_name')
+        project_domain_id = default_swift_reference.get('project_domain_id')
+        project_domain_name = default_swift_reference.get(
+            'project_domain_name')
+
+        # create client for multitenant user(trustor)
+        trustor_auth = ks_v3.Token(auth_url=auth_address,
+                                   token=context.auth_token,
+                                   project_id=context.tenant)
+        trustor_sess = ks_session.Session(auth=trustor_auth)
+        trustor_client = ks_client.Client(session=trustor_sess)
+        auth_ref = trustor_client.session.auth.get_auth_ref(trustor_sess)
+        roles = [t['name'] for t in auth_ref['roles']]
+
+        # create client for trustee - glance user specified in swift config
+        tenant_name, user = user.split(':')
+        password = ks_v3.Password(auth_url=auth_address,
+                                  username=user,
+                                  password=key,
+                                  project_name=tenant_name,
+                                  user_domain_id=user_domain_id,
+                                  user_domain_name=user_domain_name,
+                                  project_domain_id=project_domain_id,
+                                  project_domain_name=project_domain_name)
+        trustee_sess = ks_session.Session(auth=password)
+        trustee_client = ks_client.Client(session=trustee_sess)
+
+        # request glance user id - we will use it as trustee user
+        trustee_user_id = trustee_client.session.get_user_id()
+
+        # create trust for trustee user
+        trust_id = trustor_client.trusts.create(
+            trustee_user=trustee_user_id, trustor_user=context.user,
+            project=context.tenant, impersonation=True,
+            role_names=roles
+        ).id
+        # initialize a new client with trust and trustee credentials
+        # create client for glance trustee user
+        client_password = ks_v3.Password(
+            auth_url=auth_address,
+            username=user,
+            password=key,
+            trust_id=trust_id,
+            user_domain_id=user_domain_id,
+            user_domain_name=user_domain_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name
+        )
+        # now we can authenticate against KS
+        # as trustee of user who provided token
+        client_sess = ks_session.Session(auth=client_password)
+        return ks_client.Client(session=client_sess)
+
 
 class ChunkReader(object):
     def __init__(self, fd, checksum, total, verifier=None):
@@ -974,3 +1280,34 @@ class ChunkReader(object):
         if self.verifier:
             self.verifier.update(result)
         return result
+
+
+def get_manager_for_store(store, store_location,
+                          context=None,
+                          allow_reauth=False):
+    """Return appropriate connection manager for store
+
+    The method detects store type (singletenant or multitenant) and returns
+    appropriate connection manager (singletenant or multitenant) that allows
+    to request swiftclient connections.
+    :param store: store that needs swift connections
+    :param store_location: StoreLocation object that define image location
+    :param context: user context
+    :param allow_reauth: defines if we allow re-authentication when user token
+    is expired and refresh swift connection
+    :return: connection manager for store
+    """
+    if store.__class__ == SingleTenantStore:
+        return connection_manager.SingleTenantConnectionManager(
+            store, store_location, context, allow_reauth)
+    elif store.__class__ == MultiTenantStore:
+        # if global toggle is turned off then do not allow re-authentication
+        # with trusts
+        if not store.conf.glance_store.swift_store_use_trusts:
+            allow_reauth = False
+        return connection_manager.MultiTenantConnectionManager(
+            store, store_location, context, allow_reauth)
+    else:
+        raise NotImplementedError(_("There is no Connection Manager "
+                                    "implemented for %s class.") %
+                                  store.__class__.__name__)
