@@ -27,23 +27,48 @@ swift_opts = [
                default="ref1",
                help=i18n._('The reference to the default swift account/backing'
                            ' store parameters to use for adding new images.')),
+    cfg.StrOpt('swift_store_auth_version', default='2',
+               help=i18n._('Version of the authentication service to use. '
+                           'Valid versions are 2 and 3 for keystone and 1 '
+                           '(deprecated) for swauth and rackspace.'),
+               deprecated_for_removal=True,
+               deprecated_reason=i18n._('Use "auth_version" in '
+                                        'swift_store_config_file.')
+               ),
     cfg.StrOpt('swift_store_auth_address',
                help=i18n._('The address where the Swift authentication '
-                           'service is listening.(deprecated)')),
+                           'service is listening.'),
+               deprecated_for_removal=True,
+               deprecated_reason=i18n._('Use "auth_address" in '
+                                        'swift_store_config_file')
+               ),
     cfg.StrOpt('swift_store_user', secret=True,
                help=i18n._('The user to authenticate against the Swift '
-                           'authentication service (deprecated)')),
+                           'authentication service.'),
+               deprecated_for_removal=True,
+               deprecated_reason=i18n._('Use "user" in '
+                                        'swift_store_config_file.')),
     cfg.StrOpt('swift_store_key', secret=True,
                help=i18n._('Auth key for the user authenticating against the '
-                           'Swift authentication service. (deprecated)')),
+                           'Swift authentication service.'),
+               deprecated_for_removal=True,
+               deprecated_reason=i18n._('Use "key" in '
+                                        'swift_store_config_file.')
+               ),
     cfg.StrOpt('swift_store_config_file', secret=True,
                help=i18n._('The config file that has the swift account(s)'
                            'configs.')),
 ]
 
+_config_defaults = {'user_domain_id': 'default',
+                    'user_domain_name': None,
+                    'project_domain_id': 'default',
+                    'project_domain_name': None}
+
 # NOTE(bourke): The default dict_type is collections.OrderedDict in py27, but
 # we must set manually for compatibility with py26
-CONFIG = configparser.SafeConfigParser(dict_type=OrderedDict)
+CONFIG = configparser.SafeConfigParser(defaults=_config_defaults,
+                                       dict_type=OrderedDict)
 LOG = logging.getLogger(__name__)
 
 
@@ -74,6 +99,11 @@ class SwiftParams(object):
             default['user'] = glance_store.swift_store_user
             default['key'] = glance_store.swift_store_key
             default['auth_address'] = glance_store.swift_store_auth_address
+            default['project_domain_id'] = 'default'
+            default['project_domain_name'] = None
+            default['user_domain_id'] = 'default'
+            default['user_domain_name'] = None
+            default['auth_version'] = glance_store.swift_store_auth_version
             return {glance_store.default_swift_reference: default}
         return {}
 
@@ -92,12 +122,25 @@ class SwiftParams(object):
                                                    reason=msg)
         account_params = {}
         account_references = CONFIG.sections()
+
         for ref in account_references:
             reference = {}
             try:
-                reference['auth_address'] = CONFIG.get(ref, 'auth_address')
-                reference['user'] = CONFIG.get(ref, 'user')
-                reference['key'] = CONFIG.get(ref, 'key')
+                for param in ('auth_address',
+                              'user',
+                              'key',
+                              'project_domain_id',
+                              'project_domain_name',
+                              'user_domain_id',
+                              'user_domain_name'):
+                    reference[param] = CONFIG.get(ref, param)
+
+                try:
+                    reference['auth_version'] = CONFIG.get(ref, 'auth_version')
+                except configparser.NoOptionError:
+                    av = self.conf.glance_store.swift_store_auth_version
+                    reference['auth_version'] = av
+
                 account_params[ref] = reference
             except (ValueError, SyntaxError, configparser.NoOptionError) as e:
                 LOG.exception(i18n._("Invalid format of swift store config"
