@@ -40,7 +40,12 @@ S3_CONF = {'s3_store_access_key': 'user',
            's3_store_host': 'localhost:8080',
            's3_store_bucket': 'glance',
            's3_store_large_object_size': 5,        # over 5MB is large
-           's3_store_large_object_chunk_size': 6}  # part size is 6MB
+           's3_store_large_object_chunk_size': 6,  # part size is 6MB
+           's3_store_enable_proxy': False,
+           's3_store_proxy_host': None,
+           's3_store_proxy_port': 8080,
+           's3_store_proxy_user': 'user',
+           's3_store_proxy_password': 'foobar'}
 
 # ensure that mpu api is used and parts are uploaded as expected
 mpu_parts_uploaded = 0
@@ -292,7 +297,7 @@ class TestStore(base.StoreBaseTest,
             conf=self.conf)
         (image_s3, image_size) = self.store.get(loc)
 
-        self.assertEqual(image_size, FIVE_KB)
+        self.assertEqual(FIVE_KB, image_size)
 
         expected_data = b"*" * FIVE_KB
         data = b""
@@ -433,6 +438,47 @@ class TestStore(base.StoreBaseTest,
             self.assertEqual(expected_s3_contents,
                              new_image_contents.getvalue())
 
+    def test_add_with_verifier(self):
+        """
+        Assert 'verifier.update' is called when verifier is provided, both
+        for multipart and for single uploads.
+        """
+        one_part_max = 6 * units.Mi
+        variations = [(FIVE_KB, 1),  # simple put   (5KB < 5MB)
+                      (5 * units.Mi, 1),  # 1 part       (5MB <= 5MB < 6MB)
+                      (one_part_max, 1),  # 1 part exact (5MB <= 6MB <= 6MB)
+                      (one_part_max + one_part_max // 2, 2),  # 1.5 parts
+                      (one_part_max * 2, 2)]  # 2 parts exact
+
+        for (s3_size, update_calls) in variations:
+            image_id = str(uuid.uuid4())
+            base_byte = b"12345678"
+            s3_contents = base_byte * (s3_size // 8)
+            image_s3 = six.BytesIO(s3_contents)
+            verifier = mock.MagicMock(name='mock_verifier')
+
+            # add image
+            self.store.add(image_id, image_s3, s3_size, verifier=verifier)
+
+            # confirm update called expected number of times
+            self.assertEqual(verifier.update.call_count, update_calls)
+
+            if (update_calls <= 1):
+                # the contents weren't broken into pieces
+                verifier.update.assert_called_with(s3_contents)
+            else:
+                # most calls to update should be with the max one part size
+                s3_contents_max_part = base_byte * (one_part_max // 8)
+
+                # the last call to verify.update should be with what's left
+                s3_contents_last_part = base_byte * ((s3_size - one_part_max)
+                                                     // 8)
+
+                # confirm all expected calls to update have occurred
+                calls = [mock.call(s3_contents_max_part),
+                         mock.call(s3_contents_last_part)]
+                verifier.update.assert_has_calls(calls)
+
     def test_add_host_variations(self):
         """
         Test that having http(s):// in the s3serviceurl in config
@@ -501,7 +547,6 @@ class TestStore(base.StoreBaseTest,
                 capabilities.BitMasks.WRITE_ACCESS)
         except Exception:
             return False
-        return False
 
     def test_no_access_key(self):
         """
@@ -586,3 +631,45 @@ class TestStore(base.StoreBaseTest,
     def test_calling_format_default(self):
         self.assertIsInstance(s3.get_calling_format(),
                               boto.s3.connection.SubdomainCallingFormat)
+
+    def test_image_get_with_proxy_without_host(self):
+        """Test s3 backend with unconfigured proxy connection."""
+        self.config(s3_store_enable_proxy=True)
+
+        loc = location.get_location_from_uri(
+            "s3://user:key@auth_address/glance/%s" % FAKE_UUID,
+            conf=self.conf)
+        self.assertRaises(exceptions.BadStoreConfiguration,
+                          self.store.get, loc)
+
+    def test_image_get_with_proxy(self):
+        """Test s3 get with proxy connection."""
+        self.config(s3_store_enable_proxy=True)
+        proxy_host = '127.0.0.1'
+        self.config(s3_store_proxy_host=proxy_host)
+
+        with mock.patch.object(boto.s3.connection, 'S3Connection') as m:
+            cf = s3.get_calling_format(bucket_format=None,
+                                       s3_store_bucket_url_format='subdomain')
+
+            with mock.patch.object(s3, 'get_calling_format') as gcf:
+                gcf.return_value = cf
+
+                loc = location.get_location_from_uri(
+                    "s3://user:key@auth_address/glance/%s" % FAKE_UUID,
+                    conf=self.conf)
+                self.store.get(loc)
+
+                accesskey = S3_CONF['s3_store_access_key']
+                secretkey = S3_CONF['s3_store_secret_key']
+                proxy_port = S3_CONF['s3_store_proxy_port']
+                proxy_pass = S3_CONF['s3_store_proxy_password']
+                proxy_user = S3_CONF['s3_store_proxy_user']
+
+                m.assert_called_with(accesskey, secretkey,
+                                     calling_format=cf,
+                                     is_secure=False,
+                                     proxy=proxy_host,
+                                     proxy_pass=proxy_pass,
+                                     proxy_port=proxy_port,
+                                     proxy_user=proxy_user)
